/* -*-c-*-
 *
 * $Id$
 *
 * Matrix arithmetic mod %$2^{24}$%
 *
 * (c) 2000 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * Copyright (c) 2000 Mark Wooding
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2, Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. The name of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Instead of accepting the above terms, you may redistribute and/or modify
 * this software under the terms of either the GNU General Public License,
 * or the GNU Library General Public License, published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: matrix.h,v $
 * Revision 1.2  2000/07/02 15:22:07  mdw
 * Fix licence text.  Change to matinv to mark inputs as constants.
 *
 * Revision 1.1  2000/05/21 11:28:30  mdw
 * Initial check-in.
 *
 */

#ifndef MATRIX_H
#define MATRIX_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifndef BITS_H
#  include "bits.h"
#endif

/*----- Functions provided ------------------------------------------------*/

/* --- @matmul@ --- *
 *
 * Arguments:	@uint24 *d@ = pointer to destination matrix
 *		@uint24 *a, *b@ = pointer to operand matrices
 *		@unsigned x, y, z@ = dimensions of the operand matrices
 *
 * Returns:	---
 *
 * Use:		Performs matrix multiplication mod %$2^{24}$%.  The matrix
 *		@d@ may not overlap either operand matrix.
 */

extern void matmul(uint24 */*d*/, const uint24 */*a*/, const uint24 */*b*/,
		   unsigned /*x*/, unsigned /*y*/, unsigned /*z*/);

/* --- @matinv@ --- *
 *
 * Arguments:	@uint24 *d@ = pointer to destination matrix
 *		@const uint24 *a@ = pointer to operand matrix
 *		@unsigned x, y@ = dimensions of operand matrix
 *
 * Returns:	Zero if the matrix was successfully inverted, %$-1$% if the
 *		matrix is singular.
 *
 * Use:		Computes the mod %$2^{24}$% inverse of a square matrix.
 */

extern int matinv(uint24 */*d*/, const uint24 */*a*/,
		  unsigned /*x*/, unsigned /*y*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
