/* -*-c-*-
 *
 * $Id$
 *
 * Search for invertible matrices
 *
 * (c) 2000 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * Copyright (c) 2000 Mark Wooding
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2, Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. The name of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Instead of accepting the above terms, you may redistribute and/or modify
 * this software under the terms of either the GNU General Public License,
 * or the GNU Library General Public License, published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: storin-mktab.c,v $
 * Revision 1.2  2000/07/02 15:21:20  mdw
 * Fix licence text.
 *
 * Revision 1.1  2000/05/21 11:28:30  mdw
 * Initial check-in.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include "bits.h"

#include "dsarand.h"
#include "sha.h"

#include "arith24.h"
#include "matrix.h"

/*----- Main code ---------------------------------------------------------*/

int main(int argc, char *argv[])
{
  dsarand r;
  int i, j;
  uint24 d[16], a[16];

  /* --- Make the seed string --- */

  {
    sha_ctx s;
    octet h[SHA_HASHSZ];
    const char *p = "matrix-seed-string";
    size_t sz;

    if (argv[1])
      p = argv[1];
    sz = strlen(p);
    sha_init(&s);
    sha_hash(&s, p, sz);
    sha_done(&s, h);
    dsarand_init(&r, h, sizeof(h));
    r.passes = 2;
  }

  /* --- Make a matrix --- */

  for (;;) {
    octet b[4];

    for (i = 0; i < 16; i++) {
      octet w[3];
      dsarand_fill(&r, w, sizeof(w));
      a[i] = LOAD24(w);
    }

    /* --- Ensure the LSBs have appropriate properties --- */

    for (i = 0; i < 4; i++) {
      int bb = -1;
      for (j = 0; j < 4; j++) {
	if ((a[i * 4 + j] & 1) == 0) {
	  if (bb == -1)
	    bb = j;
	  else
	    goto reject;
	}
      }
      if (bb == -1)
	goto reject;
      for (j = 0; j < i; j++) {
	if (b[j] == bb)
	  goto reject;
      }
      b[i] = bb;
    }

    /* --- Ensure the matrix is invertible (and invert it) --- */

    if (matinv(d, a, 4, 4) == 0)
      break;
  reject:;
  }

  fputs("\
/* -*-c-*-\n\
 *\n\
 * Tables for Storin [generated]\n\
 */\n\
\n\
#ifndef STORIN_TAB_H\n\
#define STORIN_TAB_H\n\
\n\
#define STORIN_M {							\\\n\
  ", stdout);

  for (i = 0; i < 16; i++) {
    printf("0x%06x", a[i]);
    if (i == 15)
      fputs("				\\\n}\n\n", stdout);
    else if (i % 4 == 3)
      fputs(",				\\\n  ", stdout);
    else
      fputs(", ", stdout);
  }

  fputs("\
#define STORIN_MI {							\\\n\
  ", stdout);

  for (i = 0; i < 16; i++) {
    printf("0x%06x", d[i]);
    if (i == 15)
      fputs("				\\\n}\n\n", stdout);
    else if (i % 4 == 3)
      fputs(",				\\\n  ", stdout);
    else
      fputs(", ", stdout);
  }

  fputs("#endif\n", stdout);

  if (fclose(stdout)) {
    fputs("error writing data\n", stderr);
    exit(EXIT_FAILURE);
  }

  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
