/* -*-c-*-
 *
 * $Id$
 *
 * Generate test vectors for Storin
 *
 * (c) 2000 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * Copyright (c) 2000 Mark Wooding
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2, Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. The name of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Instead of accepting the above terms, you may redistribute and/or modify
 * this software under the terms of either the GNU General Public License,
 * or the GNU Library General Public License, published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: storin-tests.c,v $
 * Revision 1.2  2000/07/02 15:21:20  mdw
 * Fix licence text.
 *
 * Revision 1.1  2000/05/21 11:28:30  mdw
 * Initial check-in.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#include "bits.h"
#include "fibrand.h"
#include "storin.h"

/*----- Main code ---------------------------------------------------------*/

int main(void)
{
  uint24 k[8], p[4], c[4], pp[4];
  storin_ctx s;
  fibrand r;
  unsigned i, j, ii;

  fibrand_lcseed(&r, 0);

  fputs("\
# Test vectors for Storin [generated]\n\
#\n\
# In each entry, the key comes first, followed by the plaintext and\n\
# ciphertext.\n\
\n\
storin {\
", stdout);

  for (i = 1; i < 8; i++) {

    /* --- Initial plaintext value --- */

    for (j = 0; j < i; j++)
      k[j] = j + 1;
    for (j = 0; j < 4; j++)
      p[j] = i + j + 1;
    storin_init24(&s, k, i);
    storin_eblk24(&s, p, c);
    storin_dblk24(&s, c, pp);
    fputs("\n  ", stdout); for (j = 0; j < i; j++) printf("%06x", k[j]);
    fputs("\n    ", stdout); for (j = 0; j < 4; j++) printf("%06x", p[j]);
    fputc(' ', stdout); for (j = 0; j < 4; j++) printf("%06x", c[j]);
    fputs(";\n", stdout);
    for (j = 0; j < 4; j++) {
      if (p[j] != pp[j]) {
	fputs("*** decryption failure\n", stderr);
	exit(EXIT_FAILURE);
      }
    }

    /* --- Random tests --- */

    for (ii = 1; ii < 16; ii++) {
      for (j = 0; j < i; j++)
	k[j] = U24(fibrand_step(&r));
      for (j = 0; j < 4; j++)
	p[j] = U24(fibrand_step(&r));
      storin_init24(&s, k, i);
      storin_eblk24(&s, p, c);
      fputs("\n  ", stdout); for (j = 0; j < i; j++) printf("%06x", k[j]);
      fputs("\n    ", stdout); for (j = 0; j < 4; j++) printf("%06x", p[j]);
      fputc(' ', stdout); for (j = 0; j < 4; j++) printf("%06x", c[j]);
      fputs(";\n", stdout);
    }
  }

  fputs("}\n", stdout);
  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
