/* -*-c-*-
 *
 * $Id$
 *
 * Block cipher optimized for DSPs
 *
 * (c) 2000 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * Copyright (c) 2000 Mark Wooding
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2, Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. The name of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Instead of accepting the above terms, you may redistribute and/or modify
 * this software under the terms of either the GNU General Public License,
 * or the GNU Library General Public License, published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: storin.h,v $
 * Revision 1.2  2000/07/02 15:21:20  mdw
 * Fix licence text.
 *
 * Revision 1.1  2000/05/21 11:28:30  mdw
 * Initial check-in.
 *
 */

#ifndef STORIN_H
#define STORIN_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifndef BITS_H
#  include "bits.h"
#endif

/*----- Magic numbers -----------------------------------------------------*/

#define STORIN_BLKSZ 12
#define STORIN_KEYSZ 12
#define STORIN_CLASS (N, L, 96)

#define STORIN_ROUNDS 8

/*----- Data structures ---------------------------------------------------*/

typedef struct storin_ctx {
  uint24 k[4 * (STORIN_ROUNDS + 1)];
} storin_ctx;

/*----- Functions provided ------------------------------------------------*/

/* --- @storin_init24@ --- *
 *
 * Arguments:	@storin_ctx *k@ = pointer to cipher context to initialize
 *		@const uint24 *buf@ = pointer to buffer of key material
 *		@size_t sz@ = size of the key material
 *
 * Returns:	---
 *
 * Use:		Initializes the cipher for use.
 */

extern void storin_init24(storin_ctx */*k*/,
			  const uint24 */*buf*/, size_t /*sz*/);

/* --- @storin_eblk24@, @storin_dblk24@ --- *
 *
 * Arguments:	@const storin_ctx *k@ = pointer to cipher context
 *		@const uint24 s[4]@ = pointer to source block
 *		@uint24 d[4]@ = pointer to destination block
 *
 * Returns:	---
 *
 * Use:		Low-level block encryption and decryption.
 */

extern void storin_eblk24(const storin_ctx */*k*/,
			  const uint24 */*s*/, uint24 */*d*/);
extern void storin_dblk24(const storin_ctx */*k*/,
			  const uint24 */*s*/, uint24 */*d*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
