/* -*-c-*-
 *
 * $Id$
 *
 * Symbol table management
 *
 * (c) 1998 Straylight
 * (c) 2000 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * Copyright (c) 2000 Mark Wooding
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2, Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. The name of the authors may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 * NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * Instead of accepting the above terms, you may redistribute and/or modify
 * this software under the terms of either the GNU General Public License,
 * or the GNU Library General Public License, published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 */

/*----- Revision history --------------------------------------------------*
 *
 * $Log: sym.c,v $
 * Revision 1.2  2000/07/02 15:21:20  mdw
 * Fix licence text.
 *
 * Revision 1.1  2000/05/21 11:28:30  mdw
 * Initial check-in.
 *
 * --- Past lives (Become) --- *
 *
 * Revision 1.4  1998/01/12 16:46:28  mdw
 * Fix copyright date.
 *
 * Revision 1.3  1997/08/20  16:22:59  mdw
 * Patch memory leak.
 *
 * Revision 1.2  1997/08/04 10:24:25  mdw
 * Sources placed under CVS control.
 *
 * Revision 1.1  1997/07/21  13:47:44  mdw
 * Initial revision
 *
 */

/*----- Header files ------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bits.h"
#include "sym.h"

/*----- Tuning parameters -------------------------------------------------*/

/* --- Initial hash table size --- *
 *
 * This is the initial @mask@ value.  It must be of the form %$2^n - 1$%,
 * so that it can be used to mask of the bottom bits of a hash value.
 */

#define SYM_INITSZ 256			/* Size of a new hash table */

/* --- Maximum load factor --- *
 *
 * This parameter controls how much the table has to be loaded before the
 * table is extended.  The number of elements %$n$%, the number of bins %$b$%
 * and the limit %$l$% satisfy the relation %$n < bl$%; if a new item is
 * added to the table and this relation is found to be false, the table is
 * doubled in size.
 */

#define SYM_LIMIT(n) ((n) * 2)		/* Load factor for growing table */

/*----- CRC table ---------------------------------------------------------*/

/*****************************************************************/
/*                                                               */
/* CRC LOOKUP TABLE                                              */
/* ================                                              */
/*                                                               */
/* The following CRC lookup table was generated automagically    */
/* by `crcgen', which is based on the Rocksoft^tm Model CRC      */
/* Algorithm Table Generation Program.  The model parameters     */
/* supplied to `crcgen' were:                                    */
/*                                                               */
/*    Width   : 32 bits                                          */
/*    Poly    : 0x04C11DB7                                       */
/*    Init    : 0xFFFFFFFF                                       */
/*    XorOut  : 0xFFFFFFFF                                       */
/*    Reverse : Yes                                              */
/*    Check   : 0xCBF43926                                       */
/*                                                               */
/* For more information on the Rocksoft^tm Model CRC Algorithm,  */
/* see the document titled `A Painless Guide to CRC Error        */
/* Detection Algorithms' by Ross Williams                        */
/* (ross@@guest.adelaide.edu.au.). This document is likely to be */
/* in the FTP archive `ftp.adelaide.edu.au/pub/rocksoft'.        */
/*                                                               */
/*****************************************************************/

static uint32 crctab[256] = {
  0x00000000, 0x77073096, 0xee0e612c, 0x990951ba,
  0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
  0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
  0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
  0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de,
  0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
  0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec,
  0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
  0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
  0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
  0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940,
  0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
  0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116,
  0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
  0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
  0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
  0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a,
  0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
  0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818,
  0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
  0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
  0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
  0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c,
  0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
  0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2,
  0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
  0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
  0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
  0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086,
  0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
  0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4,
  0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
  0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
  0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
  0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8,
  0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
  0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe,
  0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
  0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
  0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
  0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252,
  0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
  0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60,
  0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
  0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
  0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
  0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04,
  0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
  0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a,
  0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
  0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
  0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
  0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e,
  0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
  0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c,
  0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
  0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
  0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
  0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0,
  0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
  0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6,
  0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
  0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
  0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
};

/*****************************************************************/
/*                   End of CRC Lookup Table                     */
/*****************************************************************/

/* --- Work out the CRC of a bytestring --- */

#define CRC(_s, _l, _hv) do {						\
  uint32 _h = 0xffffffff;						\
  const char *_p = _s;							\
  const char *_e = (_s) + (_l);						\
									\
  while (_p < _e)							\
    _h = (_h >> 8) ^ (crctab[(*_p++ ^ _h) & 0xFF]);			\
  _hv = ~_h & 0xffffffff;						\
} while (0)

/*----- Memory allocation -------------------------------------------------*/

static void die(const char *p)
{
  fprintf(stderr, "%s\n", p);
  exit(EXIT_FAILURE);
}

static void *xmalloc(size_t sz)
{
  void *p = malloc(sz);
  if (!p)
    die("not enough memory");
  return (p);
}

static void *xrealloc(void *p, size_t sz)
{
  p = realloc(p, sz);
  if (!p)
    die("not enough memory");
  return (p);
}


/*----- Main code ---------------------------------------------------------*/

/* --- @sym_create@ --- *
 *
 * Arguments:	@sym_table *t@ = symbol table to initialize
 *
 * Returns:	---
 *
 * Use:		Initialises the given symbol table.
 */

void sym_create(sym_table *t)
{
  size_t i;

  t->mask = SYM_INITSZ - 1;
  t->c = SYM_LIMIT(SYM_INITSZ);
  t->a = xmalloc((t->mask + 1) * sizeof(sym_base *));

  for (i = 0; i < SYM_INITSZ + 1; i++)
    t->a[i] = 0;  
}

/* --- @sym_destroy@ --- *
 *
 * Arguments:	@sym_table *t@ = pointer to symbol table in question
 *
 * Returns:	---
 *
 * Use:		Destroys a symbol table, freeing all the memory it used to
 *		occupy.
 */

void sym_destroy(sym_table *t)
{
  size_t i;
  sym_base *p, *q;

  for (i = 0; i <= t->mask; i++) {
    p = t->a[i];
    while (p) {
      q = p->next;
      free(p->name);
      free(p);
      p = q;
    }
  }
  free(t->a);
}

/* --- @sym_find@ --- *
 *
 * Arguments:	@sym_table *t@ = pointer to symbol table in question
 *		@const char *n@ = pointer to symbol table to look up
 *		@long l@ = length of the name string or negative to measure
 *		@size_t sz@ = size of desired symbol object, or zero
 *		@unsigned *f@ = pointer to a flag, or null.
 *
 * Returns:	The address of a @sym_base@ structure, or null if not found
 *		and @sz@ is zero.
 *
 * Use:		Looks up a symbol in a given symbol table.  The name is
 *		passed by the address of its first character.  The length
 *		may be given, in which case the name may contain arbitrary
 *		binary data, or it may be given as a negative number, in
 *		which case the length of the name is calculated as
 *		@strlen(n) + 1@.
 *
 *		The return value is the address of a pointer to a @sym_base@
 *		block (which may have other things on the end, as above).  If
 *		the symbol could be found, the return value points to the
 *		symbol block.  If the symbol wasn't there, then if @sz@ is
 *		nonzero, a new symbol is created and its address is returned;
 *		otherwise a null pointer is returned.
 *
 *		The value of @*f@ indicates whether a new symbol entry was
 *		created: a nonzero value indicates that an old value was
 *		found.
 */

void *sym_find(sym_table *t, const char *n, long l, size_t sz, unsigned *f)
{
  uint32 hash;
  size_t len = l < 0 ? strlen(n) + 1 : l;
  sym_base *bin;
  sym_base *p, *q;

  /* --- Find the correct bin --- */

  CRC(n, len, hash);
  bin = p = (sym_base *)(t->a + (hash & t->mask));

  /* --- Search the bin list --- */

  while (p->next) {
    if (hash == p->next->hash && len == p->next->len &&
	!memcmp(n, p->next->name, len)) {

      /* --- Found a match --- *
       *
       * As a minor, and probably pointless, tweak, move the item to the
       * front of its bin list.
       */

      q = p->next;
      p->next = q->next;
      q->next = bin->next;
      bin->next = q;

      /* --- Return the block --- */

      if (f) *f = 1;
      return (q);
    }

    p = p->next;
  }

  /* --- Couldn't find the item there --- */

  if (f) *f = 0;
  if (!sz) return (0);

  /* --- Create a new symbol block and initialize it --- */

  p = xmalloc(sz);
  p->next = bin->next;
  p->hash = hash;
  p->name = xmalloc(len);
  p->len = len;
  memcpy(p->name, n, len);

  bin->next = p;

  /* --- Consider growing the array --- */

  if (!--t->c) {
    uint32 m = t->mask + 1;
    sym_base *p, *q, *r;
    size_t i, lim;

    /* --- Update values in the anchor block --- */

    t->c = SYM_LIMIT(t->mask + 1);
    t->mask = (t->mask + 1) * 2 - 1;
    t->a = xrealloc(t->a, (t->mask + 1) * sizeof(sym_base *));

    /* --- Now wander through the table rehashing things --- *
     *
     * This loop is very careful to avoid problems with aliasing.  The items
     * are dealt with from the end backwards to avoid overwriting bins before
     * they've been processed.
     */

    lim = (t->mask + 1) >> 1;
    for (i = 0; i < lim; i++) {

      /* --- Some initialization --- */

      r = t->a[i];
      p = (sym_base *)(t->a + i);
      q = (sym_base *)(t->a + i + lim);

      /* --- Now go through the @r@ list --- */

      while (r) {
	if (r->hash & m)
	  q = q->next = r;
	else
	  p = p->next = r;
	r = r->next;
      }
      p->next = q->next = 0;
    }
  }

  /* --- Finished that, so return the new symbol block --- */

  return (p);
}

/* --- @sym_remove@ --- *
 *
 * Arguments:	@sym_table *i@ = pointer to a symbol table object
 *		@void *b@ = pointer to symbol table entry
 *
 * Returns:	---
 *
 * Use:		Removes the object from the symbol table.  The space occupied
 *		by the object and its name is freed; anything else attached
 *		to the entry should already be gone by this point.
 */

void sym_remove(sym_table *t, void *b)
{
  /* --- A quick comment --- *
   *
   * Since the @sym_base@ block contains the hash, finding the element in the
   * bin list is really quick -- it's not worth bothering with things like
   * doubly linked lists.
   */

  sym_base *p = b;
  sym_base *bin = (sym_base *)(t->a + (p->hash & t->mask));

  /* --- Find the item in the bin list --- */

  while (bin->next) {
    if (bin->next == p)
      break;
    bin = bin->next;
  }
  if (!bin->next)
    return;

  /* --- Now just remove the item from the list and free it --- *
   *
   * Oh, and bump the load counter.
   */

  bin->next = p->next;
  free(p->name);
  free(p);
  t->c++;
}

/* --- @sym_mkiter@ --- *
 *
 * Arguments:	@sym_iter *i@ = pointer to an iterator object
 *		@sym_table *t@ = pointer to a symbol table object
 *
 * Returns:	---
 *
 * Use:		Creates a new symbol table iterator which may be used to
 *		iterate through a symbol table.
 */

void sym_mkiter(sym_iter *i, sym_table *t)
{
  i->t = t;
  i->i = 0;
  i->n = 0;
}

/* --- @sym_next@ --- *
 *
 * Arguments:	@sym_iter *i@ = pointer to iterator object
 *
 * Returns:	Pointer to the next symbol found, or null when finished.
 *
 * Use:		Returns the next symbol from the table.  Symbols are not
 *		returned in any particular order.
 */

void *sym_next(sym_iter *i)
{
  sym_base *p;

  /* --- Find the next item --- */

  while (!i->n) {
    if (i->i > i->t->mask)
      return (0);
    i->n = i->t->a[i->i++];
  }

  /* --- Update the iterator block --- */

  p = i->n;
  i->n = p->next;

  /* --- Done --- */

  return (p);
}

/*----- That's all, folks -------------------------------------------------*/
